import logging
from urllib.parse import urlencode

from clockify_api_client_adgstudios.abstract_clockify import AbstractClockify


class Client(AbstractClockify):

    def __init__(self, api_key, api_url):
        super(Client, self).__init__(api_key=api_key, api_url=api_url)

    def get_clients(self, workspace_id, params=None):
        """Returns all clients.
        :param workspace_id Id of workspace to look for clients.
        :param params       URL params of request.
        :return             List of clients(dict objects).
        """
        try:
            if params:
                url_params = urlencode(params, doseq=True)
                url = self.base_url + '/workspaces/' + workspace_id + '/clients&' + url_params
            else:
                url = self.base_url + '/workspaces/' + workspace_id + '/clients/'
            return self.get(url)
        except Exception as e:
            logging.error("API error: {0}".format(e))
            raise e

    def add_client(self, workspace_id, client_name):
        """Add new client into workspace.
        :param workspace_id Id of workspace.
        :param client_name name of client
        :return             Dictionary representation of new project.
        """
        try:
            url = self.base_url + '/workspaces/' + workspace_id + '/clients/'
            data = {
                'name': client_name
            }
            return self.post(url, data)
        except Exception as e:
            logging.error("API error: {0}".format(e))
            raise e



import logging
from urllib.parse import urlencode

from clockify_api_client_adgstudios.abstract_clockify import AbstractClockify


class User(AbstractClockify):

    def __init__(self, api_key, api_url):
        super(User, self).__init__(api_key=api_key, api_url=api_url)

    def get_current_user(self):
        """Get user by paired with API key.
        :return User dictionary representation.
        """
        try:
            url = self.base_url + '/user/'
            return self.get(url)
        except Exception as e:
            logging.error("API error: {0}".format(e))
            raise e

    def get_users(self, workspace_id, params=None):
        """Returns list of all users in given workspace.
        :param workspace_id Id of workspace.
        :param params       Request URL query params.
        :return             List of Users dictionary representation.
        """
        try:
            if params:
                params = urlencode(params, doseq=True)
                url = self.base_url + '/workspaces/' + workspace_id + '/users?' + params
            else:
                url = self.base_url + '/workspaces/' + workspace_id + '/users/'
            return self.get(url)
        except Exception as e:
            logging.error("API error: {0}".format(e))
            raise e

    def add_user(self, workspace_id, email):
        """Adds new user into workspace.
        :param workspace_id Id of workspace.
        :param email        Email of new user.
        :return             Dictionary representation of user."""
        try:
            url = self.base_url + '/workspaces/' + workspace_id + '/users/'
            emails = list()
            emails.append(email)
            data = {'emails': emails}
            return self.post(url, data)
        except Exception as e:
            logging.error("API error: {0}".format(e))
            raise e

    def update_user(self, workspace_id, user_id, payload):
        """Adds new user into workspace.
        :param workspace_id Id of workspace.
        :param user_id      User Id.
        :param payload      User data to update.
        :return             Dictionary representation of user.
        """
        try:
            url = self.base_url + '/workspaces/' + workspace_id + '/users/' + user_id
            return self.put(url, payload)
        except Exception as e:
            logging.error("API error: {0}".format(e))
            raise e

    def remove_user(self, workspace_id, user_id):
        """Removes user from workspace.
        :param workspace_id Id of workspace.
        :param user_id      User Id.
        """
        try:
            url = self.base_url + '/workspaces/' + workspace_id + '/users/' + user_id
            return self.delete(url)
        except Exception as e:
            logging.error("API error: {0}".format(e))
            raise e



import logging

from clockify_api_client_adgstudios.abstract_clockify import AbstractClockify


class Report(AbstractClockify):
    def __init__(self, api_key, api_url):
        super(Report, self).__init__(api_key=api_key, api_url=api_url)
        self.base_url = f'https://reports.{api_url}'.strip('/')

    def get_summary_report(self, workspace_id, payload):
        """Calls Clockify API for summary report. Returns summary report object(Dictionary)
        :param workspace_id Id of workspace for report.
        :param payload      Body of request for summary report.
        :return             Dictionary with summary report.
        """
        try:
            url = self.base_url + '/workspaces/' + workspace_id + '/reports/summary/'
            return self.post(url, payload)
        except Exception as e:
            logging.error("API error: {0}".format(e))
            raise e

    def get_detailed_report(self, workspace_id, payload):
        """Calls Clockify API for detailed report. Returns detailed report object(Dictionary)
        :param workspace_id Id of workspace for report.
        :param payload      Body of request for detailed report.
        :return             Dictionary with detailed report.
        """
        try:
            url = self.base_url + '/workspaces/' + workspace_id + '/reports/detailed/'
            return self.post(url, payload)
        except Exception as e:
            logging.error("API error: {0}".format(e))
            raise e

    def get_weekly_report(self, workspace_id, payload):
        """Calls Clockify API for weekly report. Returns weekly report object(Dictionary)
        :param workspace_id Id of workspace for report.
        :param payload      Body of request for weekly report.
        :return             Dictionary with weekly report.
        """
        try:
            url = self.base_url + '/workspaces/' + workspace_id + '/reports/weekly/'
            return self.post(url, payload)
        except Exception as e:
            logging.error("API error: {0}".format(e))
            raise e



import logging

from clockify_api_client_adgstudios.abstract_clockify import AbstractClockify


class Workspace(AbstractClockify):

    def __init__(self, api_key, api_url):
        super(Workspace, self).__init__(api_key=api_key, api_url=api_url)

    def get_workspaces(self):
        """Returns all workspaces.
        :return List of Workspaces in dictionary representation.
        """
        try:
            url = self.base_url + '/workspaces/'
            return self.get(url)
        except Exception as e:
            logging.error("API error: {0}".format(e))
            raise e



from clockify_api_client_adgstudios.factories.abstract_factory import AbstractFactory
from clockify_api_client_adgstudios.models.task import Task


class TaskFactory(AbstractFactory):
    class Meta:
        model = Task

    api_key = None



from clockify_api_client_adgstudios.factories.abstract_factory import AbstractFactory
from clockify_api_client_adgstudios.models.project import Project


class ProjectFactory(AbstractFactory):
    class Meta:
        model = Project

    api_key = None



from clockify_api_client_adgstudios.factories.abstract_factory import AbstractFactory
from clockify_api_client_adgstudios.models.time_entry import TimeEntry


class TimeEntryFactory(AbstractFactory):
    class Meta:
        model = TimeEntry

    api_key = None



from clockify_api_client_adgstudios.factories.abstract_factory import AbstractFactory
from clockify_api_client_adgstudios.models.client import Client


class ClientFactory(AbstractFactory):
    class Meta:
        model = Client



import logging
from urllib.parse import urlencode

from clockify_api_client_adgstudios.abstract_clockify import AbstractClockify


class TimeEntry(AbstractClockify):

    def __init__(self, api_key, api_url):
        super(TimeEntry, self).__init__(api_key=api_key, api_url=api_url)

    def get_time_entries(self, workspace_id, user_id, params=None):
        """Returns user time entries.
        :param workspace_id Id of workspace.
        :param user_id      Id of user.
        :param params       Request URL query params.
        :return             List with dictionary representation of time entries from clockify.
        """
        try:
            if params:
                url_params = urlencode(params, doseq=True)
                url = self.base_url + '/workspaces/' + workspace_id + '/user/' + user_id + '/time-entries?' + url_params
            else:
                url = self.base_url + '/workspaces/' + workspace_id + '/user/' + user_id + '/time-entries/'
            time_entries_list = self.get(url)
            return time_entries_list
        except Exception as e:
            logging.error("API error: {0}".format(e))
            raise e

    def get_time_entry(self, workspace_id, time_entry_id):
        """Gets specific time entry.
        :param workspace_id  Id of workspace.
        :param time_entry_id Id of time entry
        :return              Dictionary representation of time entry.
        """
        try:
            url = self.base_url + '/workspaces/' + workspace_id + '/time-entries/' + time_entry_id
            return self.get(url)
        except Exception as e:
            logging.error("API error: {0}".format(e))
            raise e

    def update_time_entry(self, workspace_id, entry_id, payload):
        """Updates time entry in Clockify with provided payload data.
        :param workspace_id Id of workspace.
        :param entry_id     Id of time entry.
        :param payload      Dictionary with payload data for update.
        :return             Updated time entry.
        """
        try:
            url = self.base_url + '/workspaces/' + workspace_id + '/time-entries/' + entry_id
            time_entry = self.put(url, payload)
            return time_entry
        except Exception as e:
            logging.error("API error: {0}".format(e))
            raise e

    def add_time_entry(self, workspace_id, user_id, payload):
        """Adds time entry in Clockify with provided payload data.
        Paid feature, workspace need to have active paid subscription.
        :param workspace_id Id of workspace.
        :param user_id      Id of workspace.
        :param payload      Dictionary with payload data for update.
        :return             Updated time entry.
        """
        try:
            url = self.base_url + '/workspaces/' + workspace_id + '/user/' + user_id + '/time-entries/'
            time_entry = self.post(url, payload)
            return time_entry
        except Exception as e:
            logging.error("API error: {0}".format(e))
            raise e



from clockify_api_client_adgstudios.factories.abstract_factory import AbstractFactory
from clockify_api_client_adgstudios.models.workspace import Workspace


class WorkspaceFactory(AbstractFactory):
    class Meta:
        model = Workspace

    api_key = None



from clockify_api_client_adgstudios.factories.project_factory import ProjectFactory
from clockify_api_client_adgstudios.factories.report_factory import ReportFactory
from clockify_api_client_adgstudios.factories.task_factory import TaskFactory
from clockify_api_client_adgstudios.factories.time_entry_factory import TimeEntryFactory
from clockify_api_client_adgstudios.factories.user_factory import UserFactory
from clockify_api_client_adgstudios.factories.workspace_factory import WorkspaceFactory
from clockify_api_client_adgstudios.factories.client_factory import ClientFactory
from clockify_api_client_adgstudios.utils import Singleton


class ClockifyAPIClient(metaclass=Singleton):
    def __init__(self):
        self.workspaces = None
        self.projects = None
        self.tasks = None
        self.time_entries = None
        self.users = None
        self.reports = None
        self.clients = None

    def build(self, api_key, api_url):
        """Builds services from available factories.
        :param api_key Clockify API key.
        :param api_url Clockify API url.
        """

        self.workspaces = WorkspaceFactory(api_key=api_key, api_url=api_url)
        self.projects = ProjectFactory(api_key=api_key, api_url=api_url)
        self.tasks = TaskFactory(api_key=api_key, api_url=api_url)
        self.time_entries = TimeEntryFactory(api_key=api_key, api_url=api_url)
        self.users = UserFactory(api_key=api_key, api_url=api_url)
        self.reports = ReportFactory(api_key=api_key, api_url=api_url)
        self.clients = ClientFactory(api_key=api_key, api_url=api_url)
        return self



from clockify_api_client_adgstudios.factories.abstract_factory import AbstractFactory
from clockify_api_client_adgstudios.models.report import Report


class ReportFactory(AbstractFactory):
    class Meta:
        model = Report

    api_key = None



import logging
from urllib.parse import urlencode

from clockify_api_client_adgstudios.abstract_clockify import AbstractClockify


class Project(AbstractClockify):

    def __init__(self, api_key, api_url):
        super(Project, self).__init__(api_key=api_key, api_url=api_url)

    def get_projects(self, workspace_id, params=None):
        """Returns projects from given workspace with applied params if provided.
        :param workspace_id Id of workspace.
        :param params       Dictionary with request parameters.
        :return             List of projects.
        """
        try:
            if params:
                url_params = urlencode(params, doseq=True)
                url = self.base_url + '/workspaces/' + workspace_id + '/projects?' + url_params
            else:
                url = self.base_url + '/workspaces/' + workspace_id + '/projects/'
            return self.get(url)
        except Exception as e:
            logging.error("API error: {0}".format(e))
            raise e

    def add_project(self, workspace_id, project_name, client_id, billable=False, public=False, color="#16407B"):
        """Add new project into workspace.
        :param workspace_id Id of workspace.
        :param project_name Name of new project.
        :param client_id    Id of client.
        :param billable     Bool flag.
        :return             Dictionary representation of new project.
        """
        try:
            url = self.base_url + '/workspaces/' + workspace_id + '/projects/'
            data = {
                'name': project_name,
                "clientId": client_id,
                "isPublic": "true" if public else "false",
                "billable": billable
            }
            return self.post(url, data)
        except Exception as e:
            logging.error("API error: {0}".format(e))
            raise e



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='clockify-api-client-adgstudios',
    version='0.1.2',
    author="Michael Bláha, Ashlin Darius Govindasamy",
    author_email="michael.blaha@eluvia.com, adg@adgstudios.co.za",
    description="Simple python API client for clockify. Inspired by https://pypi.org/project/clockify/library.",
    packages=find_packages(),
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/eluvia-com/clockify-api-aclient",
    install_requires=['requests', 'factory_boy'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    extras_require={"dev": ["twine"]}
)



import logging
from urllib.parse import urlencode

from clockify_api_client_adgstudios.abstract_clockify import AbstractClockify


class Task(AbstractClockify):

    def __init__(self, api_key, api_url):
        super(Task, self).__init__(api_key=api_key, api_url=api_url)

    def add_task(self, workspace_id, project_id, task_name, request_data=None):
        """Creates new task in Clockify.
        :param workspace_id  Id of workspace.
        :param request_data  Dictionary with request data.
        :param project_id    Id of project.
        :param task_name     Name of new task.
        :return              Dictionary with task object representation.
        """
        try:
            url = self.base_url + '/workspaces/' + workspace_id + '/projects/' + project_id + '/tasks/'
            payload = {'name': task_name, 'projectId': project_id}
            if request_data:
                payload = {**payload, **request_data}
            return self.post(url, payload)
        except Exception as e:
            logging.error("API error: {0}".format(e))
            raise e

    def update_task(self,  workspace_id, project_id, task_id, request_data=None):
        """Updates task in Clockify.
        :param workspace_id  Id of workspace.
        :param project_id    Id of project.
        :param task_id       Id of task.
        :param request_data  Dictionary with request data.
        :return              Dictionary with task object representation.
        """
        try:
            url = self.base_url + '/workspaces/' + workspace_id + '/projects/' + project_id + '/tasks/' + task_id
            return self.put(url, request_data)
        except Exception as e:
            logging.error("API error: {0}".format(e))
            raise e

    def get_tasks(self, workspace_id, project_id, params=None):
        """Gets list of tasks from Clockify.
        :param workspace_id  Id of workspace.
        :param project_id    Id of project.
        :param params        Request URL query parameters.
        :return              List with dictionaries with task object representation.
        """
        try:
            if params:
                url_params = urlencode(params)
                url = self.base_url + '/workspaces/' + workspace_id + '/projects/' + project_id + '/tasks?' + url_params
            else:
                url = self.base_url + '/workspaces/' + workspace_id + '/projects/' + project_id + '/tasks/'
            return self.get(url)

        except Exception as e:
            logging.error("API error: {0}".format(e))
            raise e

    def get_task(self, workspace_id, project_id, task_id):
        """Gets task from Clockify.
        :param workspace_id  Id of workspace.
        :param project_id    Id of project.
        :param task_id       Request URL query parameters.
        :return              List with dictionaries with task object representation.
        """
        try:
            url = self.base_url + '/workspaces/' + workspace_id + '/projects/' + project_id + '/tasks/' + task_id
            return self.get(url)
        except Exception as e:
            logging.error("API error: {0}".format(e))
            raise e



from abc import ABC

import requests


class AbstractClockify(ABC):
    def __init__(self, api_key, api_url):

        self.base_url = f'https://global.{api_url}'.strip('/')
        self.api_key = api_key
        self.header = {'X-Api-Key': self.api_key}

    def get(self, url):
        response = requests.get(url, headers=self.header)
        if response.status_code in [200, 201, 202]:
            return response.json()
        raise Exception(response.json())

    def post(self, url, payload):
        response = requests.post(url, headers=self.header, json=payload)
        if response.status_code in [200, 201, 202]:
            return response.json()
        raise Exception(response.json())

    def put(self, url, payload):
        response = requests.put(url, headers=self.header, json=payload)
        if response.status_code in [200, 201, 202]:
            return response.json()
        raise Exception(response.json())

    def delete(self, url):
        response = requests.delete(url, headers=self.header)
        if response.status_code in [200, 201, 202, 204]:
            return response.json()
        raise Exception(response.json())
